#********************************************************************************
# Start/stop qemu virtual machines                                              *
#                                                                               *
#  Author: Kai Peter, ©2016-                                                    *
# Version: 0.60                                                                 *
# License: ISC                                                                  *
#********************************************************************************
CFF="REGFILE"
ME=`basename $0`

error() {
  printf "\033[1m  error: $(eval echo \"\$$(echo ERRMSG$1\"))\033[0m"
  EC=1 ; result ; exit 1
}
usage() {
  echo "Usage: "`basename $0`" <VM> start|stop|status|config" ; exit 1
}
ERRMSG1="Seems another instance is running!"
#ERRMSG2="Usage: "`basename $0`" <VM> start|stop|status|config"
ERRMSG3="Couldn't start VM - check qemu options!"
ERRMSG4="Couldn't shutdown VM - couldn't connect to qemu monitor!"

# as of now: don't allow direct call of links ***********************************
if [ "$ME" != "qemu-init" ] ; then
  [ ! "$QEMUD" ] && echo "Invalid: use qmail-init!" && exit 1
  VMRC=$0
fi

#********************************************************************************
# There are 2 params:
#   $1 - name of the VM
#   $2 - action: start|stop|status|config

echo $1 | grep ^/ 2>&1 >/dev/null
if [ "$?" = 0 ] ; then
  [ -f "$1" ] && VMRC="$1"     # param is the .rc file
  [ ! "$VMRC" ] && echo "Invalid VM!" && exit 1
 else
  # 'grep $1-qemu.rc': the name have to be '<name>-qemu.rc'
  VMRC=$(grep $1-qemu.rc "$CFF" 2>/dev/null)
  VMRC=$(echo $VMRC | cut -d: -f1)
  [ ! "$VMRC" ] && echo "Not a registered VM!" && exit 1
fi

[ ! $2 ] && usage
[ $2 = "start" ] || [ $2 = "stop" ] || [ $2 = "status" ] || [ $2 = "config" ] || usage

WORKDIR=`dirname $VMRC` ; cd "$WORKDIR"     #
CFGFILE=$(echo `basename $VMRC` | cut -d. -f1).cfg
[ -f "$CFGFILE" ] && . ./$CFGFILE

setports() {
  # get free port for monitor - have to be set before including config file
  [ -f /etc/qemu/monitor.conf ] && . /etc/qemu/monitor.conf || PORT=""
  [ "$PORT" = 0 ] && PORT=""         # setting PORT=0 disables monitort/vnc
  [ "$PORT" ] && MONPORT="$PORT"
  until [ ! "$MONPORT" ] || [ "$MONPORT" -gt 65000 ] ; do
    ps ax | grep "qemu-system" | grep ":$MONPORT," 2>&1 >/dev/null
    EC=$? ; [ "$EC" -ne 0 ] && break
    MONPORT=$(($MONPORT+1))
  done
  [ ! "$VNCPORT" ] && [ "$PORT" ] && VNCPORT=$(($MONPORT-$PORT))
}
[ $2 = "start" ] && setports
[ $2 = "config" ] && setports

#********************************************************************************
# build dynamic options for drives and NIC's as well control the vm's behavior
drives() {
  i=0
  until [ ! "$DSK1" ] ; do
    i=$(($i+1))
    TMP=$(eval echo \"\$$(echo DSK$i)\")   # POSIX
    [ "$TMP" ] && DRIVES="$DRIVES -drive file=$TMP,snapshot=off" || break
  done
}
cdroms() {
  i=0 ; CDROMS=""
  until [ ! "$CDR1" ] ; do
    i=$(($i+1))
    TMP=$(eval echo \"\$$(echo CDR$i)\")  # POSIX
    [ "$TMP" ] && CDROMS="$CDROMS -cdrom $TMP" || break
  done
  [ "$CDROMS" ] && DRIVES="$DRIVES $CDROMS"    # append to DRIVES if exists
}
netifs() {
  NETSCRIPTS="script=/etc/qemu/qemu-brup,downscript=/etc/qemu/qemu-brdown"
  i=0
  until [ ! "$NIC1" ] ; do
    j=$i ; i=$(($i+1))
    MAC=`hexdump -n2 -e'"DE:AD:BE:EF" 2/1 ":%02X"' <<< $(sha256sum <<< "$NAME$i")`
    NETPARAMS="netdev=net$j,mac=$MAC -netdev tap,id=net$j,$NETSCRIPTS"
    TMP=$(eval echo \"\$$(echo NIC$i)\")     # 'eval' is POSIX
    [ "$TMP" ] && NET="$NET -device $TMP,$NETPARAMS" || break
  done
}
options() {
  # consolidate all options for functions start() and config()
  [ -f "$CUSTOM" ] && . $CUSTOM
  # option "-daemonize" have to be disabled explizit by 'DAEMON=0'
  [ "$DAEMON" ] && [ "$DAEMON" = 0 ] && DAEMON="" || DAEMON="-daemonize"

  # check QEMU_DISPLAY variable
  VIDEO=""
  if [ "$QEMU_DISPLAY" ] ; then
    QEMU_DISPLAY=`echo $QEMU_DISPLAY | tr 'A-Z' 'a-z'`
    [ "$QEMU_DISPLAY" = "vnc"  ] && VIDEO="-display vnc=:$VNCPORT"
    [ "$QEMU_DISPLAY" = "none" ] && VIDEO="-display none"
  fi

  # just put the (more or less static) options from config file together
  OPTS="$CPU $DAEMON $BOOT $LOCAL $VIDEO"  # $MOUSE

  ip -6 addr 2>&1 >/dev/null   # check for IPv6
  EC=$? ; [ "$EC" = 0 ] && IPV6=1 || IPV6=0
  [ $MONPORT ] && OPTS="$OPTS -monitor tcp:localhost:$MONPORT,server,nowait,ipv4"
  [ "$IPV6" = 1 ] && [ $MONPORT ] && \
      OPTS="$OPTS -monitor tcp:localhost:$MONPORT,server,nowait,ipv6"
  # control monitor
  CTLPORT=$(($MONPORT+100))
  [ $MONPORT ] && OPTS="$OPTS -monitor tcp:localhost:$CTLPORT,server,nowait,ipv4"
  [ "$IPV6" = 1 ] && [ $MONPORT ] && \
      OPTS="$OPTS -monitor tcp:localhost:$CTLPORT,server,nowait,ipv6"

  drives ; cdroms ; netifs      # build these ones ...
  OPTS="$OPTS $DRIVES $NET"     # ... and add dynamically options ;)
  [ "$CUST_OPTS" ] && OPTS="$OPTS $CUST_OPTS"
  QEMUBIN="qemu-system-`uname -m`"
}

#********************************************************************************
check() {
  # the 'FULLNAME' have to be unique always
  [ ! -f $PIDFILE ] && EC=1 && return "$EC"
  ps ax | grep `head -1 $PIDFILE` | grep "$FULLNAME" 2>&1 >/dev/null
  EC=$?
  return "$EC"
}

result() {
  C=`tput cols` ; C=$(($C-4))   # calculate cursor position (column)
  printf "\033[${C}G"           # move cursor to column ${C} in actual line
  [ "$EC"  = "B" ] && printf "\033[1m\033[93m%s\033[0m\n" "--"  && return   # BOOTDELAY
  [ "$EC"  = "0" ] && printf "\033[32m%s\033[0m\n" "OK"
  [ "$EC" != "0" ] && printf "\033[1m\033[91m%s\033[0m\n" "!!"
}

getports() {
  # this function will be used by config() if the VM is running
  TMP=$(echo `basename $VMRC` | cut -d\- -f1)   # use the 'short' name
  PM=`ps ax | grep "$TMP" | awk '{n=split($0,a,"-");for (i=1;i<=n;i++) print a[i]}' | grep ^monitor`
  # it shows the real monitor ports ...
  MONPORT=$(echo $PM | cut -d: -f3 | cut -d, -f1)
  PV=`ps ax | grep $TMP | awk '{n=split($0,a,"-");for (i=1;i<=n;i++) print a[i]}' | grep ^display`
  # ... but calculates the VNC port
  [ "$MONPORT" ] && VNCPORT=$(($MONPORT-$PORT))
}

config() {
  check ; [ "$EC" -ne 0 ] || getports
  options
  # options 'pidfile' and 'name' differs against function 'start'
  OPTS="$OPTS -pidfile $PIDFILE -name $FULLNAME"
  TMP=`echo $OPTS | sed 's/ -/;/g' | sed 's/^-//g'`   # remove leading '-' too
  printf "Using \033[1m$VMRC\033[0m\n"
  echo $QEMUBIN
  echo $TMP | awk -F";" '{for (i=1;i<=NF;i++) printf("\t-%s\n", $i)}' <<< "$TMP"
  printf "\n\033[1m" ; status ; printf "\033[0m"
}

start() {
  check
  [ "$EC" -eq 0 ] && echo -n "$FULLNAME: " && error 1    # already running

  echo -n "Starting $FULLNAME ..."
  #******************************************************************************
  # The BOOTDELAY option will be used with qemud only.  It have to be an integer,
  # the value have to be greater than 0. Invalid values will be converted to 0.
  [ "$QEMUD" ] && declare -i INT="$BOOTDELAY"			# check for integer value
  if [ "$INT" ] && [ "$INT" -gt 0 ] ; then
    # remove $QEMUD from environment and start a new instance in background
	unset QEMUD ; sleep "$INT" && $0 "$VMRC" start >/dev/null &
	EC="B" && echo -n " (delayed)" && result		# if it fails - bummer
	exit 0
  fi

  [ "$PRECMD" ] && `$PRECMD`     # if it fails - bummer
  options
  QEMUBIN="qemu-system-`uname -m`"
  # options 'pidfile' and 'name' HAVE TO be added here explicit!
  "$QEMUBIN" $OPTS -pidfile $PIDFILE -name "$FULLNAME" #$* #1>/dev/null
  EC=$? && result
  [ "$MONPORT" ] && echo "MON=$((MONPORT))" >> "$PIDFILE"
  [ "$VNCPORT" ] && echo "VNC=$((5900+$VNCPORT))" >> "$PIDFILE"
  [ $EC -ne 0 ] && error 3
}

down() {
  TMP=$(echo `basename $VMRC` | cut -d\- -f1)   # use the 'short' name
  echo -n "Shutting down $FULLNAME ..."
  check
  [ "$EC" -ne 0 ] && result && echo "  $FULLNAME is not running!" && return
  # split the output of 'ps ax' into lines to get the port number
  S=`ps ax | grep $TMP | awk '{n=split($0,a,"-");for (i=1;i<=n;i++) print a[i]}' | grep ^monitor`

  CTLPORT=$(echo $S | cut -d: -f3 | cut -d, -f1)
  CTLPORT=$((CTLPORT+100))
  [ ! "$CTLPORT" ] && error 4 && return
  qemu-cmd -c "system_powerdown" "$TMP" 2>&1 >/dev/null
  [ "$STOPDELAY" ] && declare -i INT="$STOPDELAY"
  [ "$INT" ] && [ "$INT" -gt 0 ] && STOPDELAY="$INT" || STOPDELAY=40
  i=1   # check if VM is down (default: timeout 40 seconds)
  until [ $i -gt "$STOPDELAY" ] ; do
#	ps ax | grep ^`head -1 $PIDFILE` 2>&1 >/dev/null
	qemu-cmd -q -c "info status" "$TMP" 2>&1>/dev/null
    EC=$?
    [ "$EC" -ne 0 ] && i=$(($i+$STOPDELAY+1))
    echo -n "." ; sleep 1
    i=$(($i+1))
  done
  # we have to reverse the return codes here
  [ "$EC" = 0 ] && EC=1 || EC=0
  result $EC
  [ "$EC" = 0 ] && rm -f $PIDFILE    # remove pidfile on successful shutdown
}

status() {
  check
  [ "$EC" -eq 0 ] && echo -e "$FULLNAME \033[32mis running\033[0m with PID `head -1 $PIDFILE`" && exit
  echo -e "qemu VM $FULLNAME is \033[93mnot running\033[0m"
  [ -f "$PIDFILE" ] && rm -f "$PIDFILE"
}

#********************************************************************************
[ "$DESC" ] && FULLNAME="$NAME ($DESC)" || FULLNAME="$NAME"
# the pidfile and logfile are located in the VM's folder
PIDFILE=`pwd`/$(echo "$NAME" | tr '[A-Z]' '[a-z]')-qemu.pid
LOGFILE=`pwd`/$(echo "$NAME" | tr '[A-Z]' '[a-z]')-qemu.log

#********************************************************************************

[ "$2" = "start" ]  && start
[ "$2" = "stop" ]   && down
[ "$2" = "status" ] && status
[ "$2" = "config" ] && config

exit
