/*
 *  - added '#include <sys/types.h>' for new functions timeout{read,write}
 *  - added 'extern int ipv4socket' (thanks Erwin Hoffmann)
 *  Revision 20160728, Kai Peter
 *  - switched to 'uint_t.h'
 *  - changed 'socket_tcp' --> 'socket_tcp4', 'socket_udp' --> 'socket_udp4'
*/
#ifndef SOCKET_H
#define SOCKET_H

#include <sys/types.h>  /* need ssize_t */
#include <sys/param.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include "uint_t.h"

extern int socket_tcp4(void);
extern int socket_tcp6(void);
extern int socket_udp4(void);
extern int socket_udp6(void);
#define socket_tcp socket_tcp6
#define socket_udp socket_udp6

//extern int socket_connect4(int s, const unsigned char *ip, uint16 port);
//extern int socket_connect6(int s, const char *ip, uint16 port, uint32 scope_id);
extern int qconnect(int s, const char *ip, uint16 port, uint32 scope_id);
#define socket_connect qconnect
#define socket_connect6 socket_connect
extern int socket_connected(int);

//extern int socket_bind4(int s, const char *ip, uint16 port);
//extern int socket_bind6(int s, const char *ip, uint16 port, uint32 scope_id);
extern int socket_bind(int s, const char *ip, uint16 port, uint32 scope_id);
#define socket_bind6 socket_bind    // remoteinfo6

extern int socket_listen(int s, int backlog);
extern int socket_accept(int s, char *ip, uint16 *port, uint32 *scope_id);

//extern int socket_recv4(int,char *,int,char *,uint16 *);
ssize_t socket_recv(int s, char *buf, size_t len, const char *ip, uint16 *port, uint32 *scope_id);
//extern int socket_send4(int,const char *,int,const char *,uint16);
ssize_t socket_send(int s, const char *buf, size_t len, const char *ip, uint16 port, uint32 scope_id);

extern int qsockinfo(int s, char *ip, uint16 *port, uint32 *scope_id, size_t info);
#define socket_local(s,ip,port,scope_id)  qsockinfo((s),(ip),(port),(scope_id),0)
#define socket_remote(s,ip,port,scope_id) qsockinfo((s),(ip),(port),(scope_id),1)

/* the following is not used yet: */
/* enable sending udp packets to the broadcast address */
//extern int socket_broadcast(int);
/* join a multicast group on the given interface */
//extern int socket_mcjoin4(int,char *,char *);
//extern int socket_mcjoin6(int,char *,int);
/* leave a multicast group on the given interface */
//extern int socket_mcleave4(int,char *);
//extern int socket_mcleave6(int,char *);
/* set multicast TTL/hop count for outgoing packets */
//extern int socket_mcttl4(int,char);
//extern int socket_mcttl6(int,char);
/* enable multicast loopback */
//extern int socket_mcloop4(int,char);
//extern int socket_mcloop6(int,char);

extern const char* socket_getifname(uint32 interface);
extern uint32 socket_getifidx(const char *ifname);

extern int ipv4socket;

//extern int socket_reuse(int s, const char *ip, uint16 port, uint32 scope_id);
extern int socket_reuse(int s);
//#define socket_bind_reuse socket_reuse  // DON'T USE! function socket_reuse changed!!
// devel: test with tcpserver
#define socket_bind_reuse2(s,ip,p,scope_id) (socket_reuse((s)), socket_bind((s),(ip),(p),(scope_id)))
//extern int socket_ipoptionskill(int);
//extern int socket_ip6anycast(int);
extern int socket_tcpnodelay(int);
//extern void socket_tryreservein(int,int);

extern int timeoutconn(int fd, char *ip, uint16 port, unsigned int timeout, uint32 scope_id);
extern ssize_t timeoutread(int fd, char *buf, size_t len, unsigned int timeout);
extern ssize_t timeoutwrite(int fd, char *buf, size_t len, unsigned int timeout);

// FreeBSD
//ssize_t recvfrom();
//ssize_t sendto();

#endif
